from base_st_classes import *

class CloseSuite(NFSSuite):
    """Test operation 4: CLOSE
    """

    def setUp(self):
        NFSSuite.setUp(self)
        self.fh = None
        self.stateid = None
        self.init_connection()

    def tearDown(self):
        self.ncl.close_file(self.fh, self.stateid, error=None)
        try: self.clean_dir(self.tmp_dir)
        except SkipException, e:
            pass

    #
    # Tests that should succeed
    #
    def testCloseCreate(self):
        """CLOSE a normal created file
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        self.ncl.close_file(self.fh, self.stateid)
        self.fh = self.stateid = None

    def testCloseOpen(self):
        """CLOSE a normal file which was opened without creation
        """
        self.fh, self.stateid = self.ncl.open_confirm(self.regfile[:-1],
                                                      self.regfile[-1])
        self.ncl.close_file(self.fh, self.stateid)
        self.fh = self.stateid = None

    #
    # Tests that may or may not work depending on server
    #
    def testCloseLocked(self):
        """CLOSE a file with outstanding locks
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        self.ncl.lock_file(self.fh, self.stateid)
        self.ncl.close_file(self.fh, self.stateid,
                            error=[NFS4_OK, NFS4ERR_LOCKS_HELD])
        self.fh = self.stateid = None

    #
    # Tests for various error conditions
    #
    def testBadSeqid(self):
        """CLOSE with a bad sequence should return NFS4ERR_BAD_SEQID
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        seqid = self.ncl.seqid + 2
        self.ncl.close_file(self.fh, self.stateid, seqid,
                            error=[NFS4ERR_BAD_SEQID])

    def testBadStateid(self):
        """CLOSE should return NFS4ERR_BAD_STATEID if use a bad id
        """
        self.fh, self.stateid = self.ncl.create_confirm()
        stateid = stateid4(0, "")
        seqid = self.ncl.seqid
        self.ncl.close_file(self.fh, stateid, error=[NFS4ERR_BAD_STATEID])
        self.ncl.seqid = seqid
        
    def testOldStateid(self):
        """CLOSE with old stateid should return NFS4ERR_OLD_STATEID
        """

        # Open a file
        dir = self.regfile[0:-1]
        name = self.regfile[-1]
        ops = [self.ncl.putrootfh_op()] + self.ncl.lookup_path(dir)
        ops.append(self.ncl.open(name))
        ops.append(self.ncl.getfh_op())
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)

        fh = res.resarray[-1].arm.arm.object
        stateid = res.resarray[-2].arm.arm.stateid

        # Confirm the open
        ops = [self.ncl.putfh_op(fh)]
        confirmop = self.ncl.open_confirm_op(stateid, self.ncl.get_seqid())
        ops.append(confirmop)
        res = self.ncl.do_ops(ops)
        self.assert_OK(res)
        
        # Close the file with old stateid
        self.ncl.close_file(self.fh, stateid, error=[NFS4ERR_OLD_STATEID])

    def testNoCfh(self):
        """CLOSE with no (cfh) should return NFS4ERR_NOFILEHANDLE
        """
        # Open the file
        self.fh, self.stateid = self.ncl.create_confirm()

        # Close the file without setting (cfh)
        ops = [self.ncl.close_op((self.ncl.seqid+1)%2**32L, self.stateid)]
        res = self.ncl.do_ops(ops)
        self.assert_status(res,[NFS4ERR_NOFILEHANDLE])

    def testTimedoutClose(self):
        """CLOSE: Try to close file after timed out with locks held

        This returns BAD_STATEID, is this correct?
        """
        #FRED - correct return? is NFS4_OK ok too?
        sleeptime = self.getLeaseTime() * 3 / 2
        self.fh, self.stateid = self.ncl.create_confirm()
        lockid = self.ncl.lock_file(self.fh, self.stateid)
        self.sleep(sleeptime)
        self.ncl.close_file(self.fh, self.stateid,
                            error=[NFS4ERR_BAD_STATEID, NFS4_OK])
        self.fh = self.stateid = None

